@@ -19,7 +19,7 @@
  *
  * input plugin for http network streams
  *
- * $Id: input_http.c,v 1.110 2005/05/21 16:16:35 jstembridge Exp $
+ * $Id: input_http.c,v 1.124 2006/11/30 10:54:18 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -176,6 +176,16 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
     return 1;
   }
   if (!info->h_name) return 1;
+  
+  if ( info->h_addr_list[0] ) {
+    /* \177\0\0\1 is the *octal* representation of 127.0.0.1 */
+    if ( info->h_addrtype == AF_INET && !memcmp(info->h_addr_list[0], "\177\0\0\1", 4) ) {
+      lprintf("host '%s' is localhost\n", host);
+      return 1;
+    }
+    /* TODO: IPv6 check */
+  }
+
   target = info->h_name;
 
   host_len = strlen(target);
@@ -286,7 +296,7 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
   xine_ui_data_t data;
     
   /* get the length of the metadata */
-  if (_x_io_tcp_read (this->stream, this->fh, &len, 1) != 1)
+  if (_x_io_tcp_read (this->stream, this->fh, (char*)&len, 1) != 1)
     return 0;
 
   lprintf ("http_plugin_read_metainf: len=%d\n", len);
@@ -300,9 +310,14 @@ static int http_plugin_read_metainf (http_input_plugin_t *this) {
     lprintf ("http_plugin_read_metainf: %s\n", metadata_buf);
 
     /* Extract the title of the current song */
-    if ((songtitle = strstr(metadata_buf, "StreamTitle='"))) {
-      songtitle += 13; /* skip "StreamTitle='" */
-      if ((title_end = strchr(songtitle, '\''))) {
+    if ((songtitle = strstr(metadata_buf, "StreamTitle="))) {
+      char terminator[] = { ';', 0, 0 };
+      songtitle += 12; /* skip "StreamTitle=" */
+      if (*songtitle == '\'' || *songtitle == '"') {
+        terminator[0] = *songtitle++;
+        terminator[1] = ';';
+      }
+      if ((title_end = strstr(songtitle, terminator))) {
         *title_end = '\0';
         
         if ((!this->shoutcast_songtitle ||
@@ -434,7 +449,7 @@ static int resync_nsv(http_input_plugin_t *this) {
   lprintf("resyncing NSV stream\n");
   while ((pos < 3) && (read_bytes < (1024*1024))) {
   
-    if (http_plugin_read_int(this, &c, 1) != 1)
+    if (http_plugin_read_int(this, (char*)&c, 1) != 1)
       return 1;
 
     this->preview[pos] = c;
@@ -481,7 +496,7 @@ static buf_element_t *http_plugin_read_block (input_plugin_t *this_gen, fifo_buf
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
 
-  total_bytes = http_plugin_read (this_gen, buf->content, todo);
+  total_bytes = http_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -627,7 +642,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   http_input_class_t  *this_class = (http_input_class_t *) this->input_plugin.input_class;
   int                  done, len, linenum;
   int                  httpcode;
-  int                  res, progress;
+  int                  res;
   int                  buflen;
   int                  use_proxy;
   int                  proxyport;
@@ -657,10 +672,10 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   if (this->port == 0)
     this->port = DEFAULT_HTTP_PORT;
   
-  if ((this->user && strlen(this->user)) && use_proxy) {
+  if (this->user && strlen(this->user)) {
     if (http_plugin_basicauth (this->user, this->password, this->auth, BUFSIZE)) {
       _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "basic auth error", NULL);
-      return 0;
+      return -1;
     }
   }
   
@@ -694,18 +709,30 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->curpos = 0;
   
   if (this->fh == -1)
-    return 0;
-  
-  /* connection timeout 20s */
-  progress = 0;
-  do {
-    report_progress(this->stream, progress);
-    res = _x_io_select (this->stream, this->fh, XIO_WRITE_READY, 500);
-    progress += 2;
-  } while ((res == XIO_TIMEOUT) && (progress < 100));
-  if (res != XIO_READY)
-    return 0;
-  
+    return -2;
+
+  {  
+    uint32_t         timeout, progress;
+    xine_cfg_entry_t cfgentry;
+    if (xine_config_lookup_entry (this->stream->xine, "media.network.timeout", &cfgentry)) {
+      timeout = cfgentry.num_value * 1000;
+    } else {
+      timeout = 30000; /* 30K msecs = 30 secs */
+    }
+
+    progress = 0;
+    do {
+      report_progress(this->stream, progress);
+      res = _x_io_select (this->stream, this->fh, XIO_WRITE_READY, 500);
+      progress += (500*100000)/timeout;
+    } while ((res == XIO_TIMEOUT) && (progress <= 100000));
+
+    if (res != XIO_READY) {
+      _x_message(this->stream, XINE_MSG_NETWORK_UNREACHABLE, this->mrl, NULL);
+      return -3;
+    }
+  }
+
   if (use_proxy) {
     if (this->port != DEFAULT_HTTP_PORT) {
       snprintf (this->buf, BUFSIZE, "GET http://%s:%d%s HTTP/1.0\015\012",
@@ -750,7 +777,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   if (_x_io_tcp_write (this->stream, this->fh, this->buf, buflen) != buflen) {
     _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "couldn't send request", NULL);
     xprintf(this_class->xine, XINE_VERBOSITY_DEBUG, "input_http: couldn't send request\n");
-    return 0;
+    return -4;
   }
 
   lprintf ("request sent: >%s<\n", this->buf);
@@ -760,13 +787,10 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->contentlength = 0;
 
   while (!done) {
-
-    /* 
-       printf ("input_http: read...\n");
-    */
+    /* fprintf (stderr, "input_http: read...\n"); */
 
     if (_x_io_tcp_read (this->stream, this->fh, &this->buf[len], 1) <= 0) {
-      return 0;
+      return -5;
     }
 
     if (this->buf[len] == '\012') {
@@ -795,7 +819,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	    _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "invalid http answer", NULL);
 	    xine_log (this->stream->xine, XINE_LOG_MSG, 
 		      _("input_http: invalid http answer\n"));
-	    return 0;
+	    return -6;
 	  }
 	}
 
@@ -808,20 +832,20 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		                        httpcode, httpstatus);
-	  return 0;
+	  return -7;
 	} else if (httpcode == 403) {
           _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
 	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		    httpcode, httpstatus);
-	  return 0;
+	  return -8;
 	} else if (httpcode < 200 || httpcode >= 300) {
 	  _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "http status not 2xx: ",
 	               httpstatus, NULL);
       	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		    httpcode, httpstatus);
-	  return 0;
+	  return -9;
 	}
       } else {
 	if (this->contentlength == 0) {
@@ -895,6 +919,12 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 	len = 0;
     } else
       len ++;
+    if ( len >= BUFSIZE ) {
+       _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
+       xine_log (this->stream->xine, XINE_LOG_MSG,
+         _("input_http: buffer exhausted after %d bytes."), BUFSIZE);
+       return -10;
+    }
   }
 
   lprintf ("end of headers\n");
@@ -905,7 +935,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   this->preview_size = MAX_PREVIEW_SIZE;
   if (this->is_nsv) {
     if (!resync_nsv(this))
-      return 0;
+      return -11;
     
     /* the first 3 chars are "NSV" */
     this->preview_size = http_plugin_read_int (this, this->preview + 3, MAX_PREVIEW_SIZE - 3);
@@ -914,7 +944,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   }
   if (this->preview_size < 0) {
     xine_log (this->stream->xine, XINE_LOG_MSG, _("input_http: read error %d\n"), errno);
-    return 0;
+    return -12;
   }
   
   lprintf("preview_size=%lld\n", this->preview_size);
@@ -931,15 +961,23 @@ static input_plugin_t *http_class_get_instance (input_class_t *cls_gen, xine_str
   /* http_input_class_t  *cls = (http_input_class_t *) cls_gen;*/
   http_input_plugin_t *this;
   
-  if (strncasecmp (mrl, "http://", 7) && (strncasecmp (mrl, "unsv://", 7))) {
+  if (strncasecmp (mrl, "http://", 7) && 
+      strncasecmp (mrl, "unsv://", 7) &&
+      strncasecmp (mrl, "peercast://pls/", 15)) {
     return NULL;
   }
   this = (http_input_plugin_t *) xine_xmalloc(sizeof(http_input_plugin_t));
 
-  this->mrl     = strdup(mrl);
-  this->stream  = stream;
-  this->fh      = -1;
-  this->nbc     = nbc_init (this->stream);
+  if (!strncasecmp (mrl, "peercast://pls/", 15)) {
+    this->mrl = xine_xmalloc (30 + strlen(mrl) - 15);
+    sprintf (this->mrl, "http://127.0.0.1:7144/stream/%s", mrl+15);
+  } else {    
+    this->mrl = strdup (mrl);
+  }
+  
+  this->stream = stream;
+  this->fh     = -1;
+  this->nbc    = nbc_init (this->stream);
   
   this->input_plugin.open              = http_plugin_open;
   this->input_plugin.get_capabilities  = http_plugin_get_capabilities;
@@ -1057,8 +1095,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "http", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "http", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
