@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.90 2005/05/29 10:18:23 valtri Exp $
+ * $Id: xine_decoder.c,v 1.96 2006/09/26 02:36:55 dgp85 Exp $
  *
  */
 
@@ -339,13 +339,16 @@ static void draw_subtitle(sputext_decoder_t *this, int64_t sub_start, int64_t su
   int line, y;
   int font_size;
 
+  _x_assert(this->renderer != NULL);
+  if ( ! this->renderer )
+    return;
+
   update_font_size(this, 0);
   
   if( strcmp(this->font, this->class->font) ) {
     strncpy(this->font, this->class->font, FONTNAME_SIZE);
     this->font[FONTNAME_SIZE - 1] = '\0';
-    if( this->renderer )
-      this->renderer->set_font (this->osd, this->class->font, this->font_size);
+    this->renderer->set_font (this->osd, this->class->font, this->font_size);
   }
 
   font_size = this->font_size;
@@ -696,7 +699,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
     _x_get_current_info (this->stream->master, &extra_info, sizeof(extra_info) );
    
-    lprintf("master: %d slave: %d input_normpos: %lld vo_discard: %d\n", 
+    lprintf("master: %d slave: %d input_normpos: %d vo_discard: %d\n", 
       master_status, slave_status, extra_info.input_normpos, vo_discard);
 
     if( !this->started && (master_status == XINE_STATUS_PLAY &&
@@ -855,7 +858,17 @@ static spu_decoder_t *sputext_class_open_plugin (spu_decoder_class_t *class_gen,
   return (spu_decoder_t *) this;
 }
 
-static void sputext_class_dispose (spu_decoder_class_t *this) {
+static void sputext_class_dispose (spu_decoder_class_t *class_gen) {
+  sputext_class_t *this = (sputext_class_t *)class_gen;
+
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.src_encoding");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.subtitle_size");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.vertical_offset");
+  this->xine->config->unregister_callback(this->xine->config,
+					  "subtitles.separate.use_unscaled_osd");
   free (this);
 }
 
@@ -944,12 +957,12 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
 /* plugin catalog information */
 static uint32_t supported_types[] = { BUF_SPU_TEXT, BUF_SPU_OGM, 0 };
 
-static decoder_info_t spudec_info = {
+static const decoder_info_t spudec_info = {
   supported_types,     /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_SPU_DECODER | PLUGIN_MUST_PRELOAD, 16, "sputext", XINE_VERSION_CODE, &spudec_info, &init_spu_decoder_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
