@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2005 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with self program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_out.c,v 1.194 2005/10/30 02:18:35 miguelfreitas Exp $
+ * $Id: audio_out.c,v 1.207 2006/11/04 23:30:14 dsalt Exp $
  *
  * 22-8-2001 James imported some useful AC3 sections from the previous alsa driver.
  *   (c) 2001 Andy Lo A Foe <andy@alsaplayer.org>
@@ -360,6 +360,7 @@ static audio_buffer_t *fifo_remove_int (audio_fifo_t *fifo, int blocking) {
 
       fifo->last = NULL;
       fifo->num_buffers = 0;
+      pthread_cond_signal (&fifo->empty);
 
     } else 
       fifo->num_buffers--;
@@ -410,33 +411,25 @@ static int fifo_num_buffers (audio_fifo_t *fifo) {
 static void fifo_wait_empty (audio_fifo_t *fifo) {
 
   pthread_mutex_lock (&fifo->mutex);
-  pthread_cond_signal (&fifo->not_empty);
-  pthread_cond_wait (&fifo->empty, &fifo->mutex);
+  while (fifo->first) {
+    /* i think it's strange to send not_empty signal here (beside the enqueue
+     * function), but it should do no harm. [MF] */
+    pthread_cond_signal (&fifo->not_empty);
+    pthread_cond_wait (&fifo->empty, &fifo->mutex);
+  }
   pthread_mutex_unlock (&fifo->mutex);
 }
 
 
 static void write_pause_burst(aos_t *this, uint32_t num_frames) { 
-  
-  int error = 0;
-  unsigned char buf[8192];
-  unsigned short *sbuf = (unsigned short *)&buf[0];
-  
+  uint16_t sbuf[4096];
+
   sbuf[0] = 0xf872;
   sbuf[1] = 0x4e1f;
-  
-  if (error == 0)
-    /* Audio ES Channel empty, wait for DD Decoder or pause */
-    sbuf[2] = 0x0003;
-  else
-    /* user stop, skip or error */
-    sbuf[2] = 0x0103;
-
+  /* Audio ES Channel empty, wait for DD Decoder or pause */
+  sbuf[2] = 0x0003;
   sbuf[3] = 0x0020;
-  sbuf[4] = 0x0000;
-  sbuf[5] = 0x0000;
-
-  memset(&sbuf[6], 0, 6144 - 96);
+  memset(&sbuf[4], 0, sizeof(sbuf) - 4 * sizeof(uint16_t));
   while (num_frames > 1536) {
     pthread_mutex_lock( &this->driver_lock );
     if(this->driver_open)
@@ -444,7 +437,6 @@ static void write_pause_burst(aos_t *this, uint32_t num_frames) {
     pthread_mutex_unlock( &this->driver_lock );
     num_frames -= 1536;
   }
-
 }
 
 
@@ -516,6 +508,24 @@ int _x_ao_mode2channels( int mode ) {
   return 0;
 } 
 
+int _x_ao_channels2mode( int channels ) {
+
+  switch( channels ) {
+    case 1:
+      return AO_CAP_MODE_MONO;
+    case 2:
+      return AO_CAP_MODE_STEREO;
+    case 3:
+    case 4:
+      return AO_CAP_MODE_4CHANNEL;
+    case 5:
+      return AO_CAP_MODE_5CHANNEL;
+    case 6:
+      return AO_CAP_MODE_5_1CHANNEL;
+  }
+  return AO_CAP_NOCAP;
+}
+
 static void audio_filter_compress (aos_t *this, int16_t *mem, int num_frames) {
 
   int    i, maxs;
@@ -615,7 +625,7 @@ static void audio_filter_amp (aos_t *this, void *buf, int num_frames) {
 static void audio_filter_equalize (aos_t *this,
 				   int16_t *data, int num_frames) {
   int       index, band, channel;
-  int       halflength, length;
+  int       length;
   int       out[EQ_CHANNELS], scaledpcm[EQ_CHANNELS];
   int64_t l;
   int       num_channels;
@@ -624,10 +634,9 @@ static void audio_filter_equalize (aos_t *this,
   if (!num_channels)
     return;
 
-  halflength = num_frames * 2;
-  length = num_frames * 4;
+  length = num_frames * num_channels;
 
-  for (index = 0; index < halflength; index+=2) {
+  for (index = 0; index < length; index += num_channels) {
 
     for (channel = 0; channel < num_channels; channel++) {
 
@@ -954,6 +963,7 @@ static void *ao_loop (void *this_gen) {
 
   last_sync_time = bufs_since_sync = 0;
   in_buf = NULL;
+  cur_time = -1;
 
   while ((this->audio_loop_running) ||
 	 (!this->audio_loop_running && this->out_fifo->first)) {
@@ -1009,6 +1019,11 @@ static void *ao_loop (void *this_gen) {
 	  in_buf = NULL;
 	  continue;
 	}
+
+        if ((in_buf->vpts - cur_time) > 2 * 90000)
+	  xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+		 "audio_out: vpts/clock error, in_buf->vpts=%" PRId64 " cur_time=%" PRId64 "\n",
+		 in_buf->vpts, cur_time);
       }
 
       lprintf ("loop:pause: I feel sleepy (%d buffers).\n", this->out_fifo->num_buffers);
@@ -1025,10 +1040,14 @@ static void *ao_loop (void *this_gen) {
          in_buf->format.rate != this->input.rate ||
          in_buf->format.mode != this->input.mode ) {
          lprintf("audio format has changed\n");
-         ao_change_settings(this,
-                            in_buf->format.bits,
-                            in_buf->format.rate,
-                            in_buf->format.mode);
+         if( !in_buf->stream->emergency_brake && 
+             ao_change_settings(this,
+                                in_buf->format.bits,
+                                in_buf->format.rate,
+                                in_buf->format.mode) == 0 ) {
+             in_buf->stream->emergency_brake = 1;
+             _x_message (in_buf->stream, XINE_MSG_AUDIO_OUT_UNAVAILABLE, NULL);
+         }
       }
     }
 
@@ -1049,12 +1068,15 @@ static void *ao_loop (void *this_gen) {
       pthread_mutex_unlock( &this->driver_lock );
       
       if (in_buf && in_buf->num_frames) {
+	xine_list_iterator_t ite;
+
 	xprintf(this->xine, XINE_VERBOSITY_LOG,
 		_("audio_out: delay calculation impossible with an unavailable audio device\n"));
 
 	pthread_mutex_lock(&this->streams_lock);
-	for (stream = xine_list_first_content(this->streams);
-	     stream; stream = xine_list_next_content(this->streams)) {
+	for (ite = xine_list_front(this->streams);
+	     ite; ite = xine_list_next(this->streams, ite)) {
+	  stream = xine_list_get_value (this->streams, ite);
           if( !stream->emergency_brake ) {
             stream->emergency_brake = 1;
             _x_message (stream, XINE_MSG_AUDIO_OUT_UNAVAILABLE, NULL);
@@ -1134,11 +1156,12 @@ static void *ao_loop (void *this_gen) {
                 cur_time > (last_sync_time + SYNC_TIME_INVERVAL) && 
                 bufs_since_sync >= SYNC_BUF_INTERVAL &&
                 !this->resample_sync_method ) {
-	xine_stream_t *stream;
+	xine_list_iterator_t *ite;
         lprintf ("audio_loop: ADJ_VPTS\n");
 	pthread_mutex_lock(&this->streams_lock);
-	for (stream = xine_list_first_content(this->streams); stream;
-	     stream = xine_list_next_content(this->streams)) {
+	for (ite = xine_list_front(this->streams); ite;
+	     ite = xine_list_next(this->streams, ite)) {
+	  xine_stream_t *stream = xine_list_get_value(this->streams, ite);
 	  if (stream == XINE_ANON_STREAM) continue;
 	  stream->metronom->set_option(stream->metronom, METRONOM_ADJ_VPTS_OFFSET,
                                        -gap/SYNC_GAP_RATE );
@@ -1230,11 +1253,13 @@ int xine_get_next_audio_frame (xine_audio_port_t *this_gen,
   lprintf ("get_next_audio_frame\n");
 
   while (!in_buf || !stream) {
-    stream = xine_list_first_content(this->streams);
-    if (!stream) {
+    xine_list_iterator_t ite = xine_list_front (this->streams);
+
+    if (!ite) {
       xine_usec_sleep (5000);
       continue;
     }
+    stream = xine_list_get_value(this->streams, ite);
   
     /* FIXME: ugly, use conditions and locks instead? */
     
@@ -1327,7 +1352,7 @@ static int ao_update_resample_factor(aos_t *this) {
 static int ao_change_settings(aos_t *this, uint32_t bits, uint32_t rate, int mode) {
   int output_sample_rate;
 
-  if(this->driver_open)
+  if(this->driver_open && !this->grab_only)
     this->driver->close(this->driver);  
   this->driver_open = 0;
 
@@ -1415,15 +1440,17 @@ static int ao_open(xine_audio_port_t *this_gen, xine_stream_t *stream,
       fifo_wait_empty(this->out_fifo);
     }
 
-    pthread_mutex_lock( &this->driver_lock );
-    ret = ao_change_settings(this, bits, rate, mode);
-    pthread_mutex_unlock( &this->driver_lock );
-
-    if( !ret ) {
-      if( !stream->emergency_brake ) {
+    if( !stream->emergency_brake ) {
+      pthread_mutex_lock( &this->driver_lock );
+      ret = ao_change_settings(this, bits, rate, mode);
+      pthread_mutex_unlock( &this->driver_lock );
+      
+      if( !ret ) {
         stream->emergency_brake = 1;
         _x_message (stream, XINE_MSG_AUDIO_OUT_UNAVAILABLE, NULL);
+        return 0;
       }
+    } else {
       return 0;
     }
   }
@@ -1445,7 +1472,7 @@ static int ao_open(xine_audio_port_t *this_gen, xine_stream_t *stream,
   }
 
   pthread_mutex_lock(&this->streams_lock);
-  xine_list_append_content(this->streams, stream);
+  xine_list_push_back(this->streams, stream);
   pthread_mutex_unlock(&this->streams_lock);
   
   return this->output.rate;
@@ -1512,23 +1539,25 @@ static void ao_put_buffer (xine_audio_port_t *this_gen,
 static void ao_close(xine_audio_port_t *this_gen, xine_stream_t *stream) {
 
   aos_t *this = (aos_t *) this_gen;
-  xine_stream_t *cur;
+  xine_list_iterator_t ite;
 
   xprintf (this->xine, XINE_VERBOSITY_DEBUG, "ao_close\n");
 
   /* unregister stream */
   pthread_mutex_lock(&this->streams_lock);
-  for (cur = xine_list_first_content(this->streams); cur;
-       cur = xine_list_next_content(this->streams))
+  for (ite = xine_list_front(this->streams); ite;
+       ite = xine_list_next(this->streams, ite)) {
+    xine_stream_t *cur = xine_list_get_value(this->streams, ite);
     if (cur == stream) {
-      xine_list_delete_current(this->streams);
+      xine_list_remove(this->streams, ite);
       break;
     }
-  cur = xine_list_first_content(this->streams);    
+  }
+  ite = xine_list_front(this->streams);    
   pthread_mutex_unlock(&this->streams_lock);
 
   /* close driver if no streams left */
-  if (!cur && !this->grab_only && !stream->gapless_switch) {
+  if (!ite && !this->grab_only && !stream->gapless_switch) {
     xprintf (this->xine, XINE_VERBOSITY_DEBUG, "audio_out: no streams left, closing driver\n");
 
     if (this->audio_loop_running) {
@@ -1584,7 +1613,7 @@ static void ao_exit(xine_audio_port_t *this_gen) {
   pthread_mutex_destroy(&this->driver_lock);
   pthread_mutex_destroy(&this->driver_action_lock);
   pthread_mutex_destroy(&this->streams_lock);
-  xine_list_free(this->streams);
+  xine_list_delete(this->streams);
 
   free (this->frame_buf[0]->mem);
   free (this->frame_buf[0]->extra_info);
@@ -1659,7 +1688,6 @@ static uint32_t ao_get_capabilities (xine_audio_port_t *this_gen) {
 
 static int ao_get_property (xine_audio_port_t *this_gen, int property) {
   aos_t *this = (aos_t *) this_gen;
-  xine_stream_t *cur;
   int ret;
 
   switch (property) {
@@ -1672,11 +1700,8 @@ static int ao_get_property (xine_audio_port_t *this_gen, int property) {
     break;
   
   case AO_PROP_NUM_STREAMS:
-    ret = 0;
     pthread_mutex_lock(&this->streams_lock);
-    for (cur = xine_list_first_content(this->streams); cur;
-         cur = xine_list_next_content(this->streams))
-      ret++;
+    ret = xine_list_size(this->streams);
     pthread_mutex_unlock(&this->streams_lock);
     break;
   
@@ -1912,10 +1937,12 @@ static int ao_status (xine_audio_port_t *this_gen, xine_stream_t *stream,
   aos_t *this = (aos_t *) this_gen;
   xine_stream_t *cur;
   int ret = 0;
+  xine_list_iterator_t ite;
             
   pthread_mutex_lock(&this->streams_lock);
-  for (cur = xine_list_first_content(this->streams); cur;
-       cur = xine_list_next_content(this->streams))
+  for (ite = xine_list_front(this->streams); ite;
+       ite = xine_list_next(this->streams, ite)) {
+    cur = xine_list_get_value(this->streams, ite);
     if (cur == stream || !stream) {
       *bits = this->input.bits;
       *rate = this->input.rate;
@@ -1923,6 +1950,7 @@ static int ao_status (xine_audio_port_t *this_gen, xine_stream_t *stream,
       ret = !!stream; /* return false for a NULL stream, true otherwise */
       break;
     }
+  }
   pthread_mutex_unlock(&this->streams_lock);
   
   return ret;        
@@ -1956,8 +1984,8 @@ xine_audio_port_t *_x_ao_new_port (xine_t *xine, ao_driver_t *driver,
   aos_t           *this;
   int              i, err;
   pthread_attr_t   pth_attrs;
-  static     char *resample_modes[] = {"auto", "off", "on", NULL};
-  static     char *av_sync_methods[] = {"metronom feedback", "resample", NULL};
+  static const char* resample_modes[] = {"auto", "off", "on", NULL};
+  static const char* av_sync_methods[] = {"metronom feedback", "resample", NULL};
 
   this = xine_xmalloc (sizeof (aos_t)) ;
 
