@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.324 2006/07/03 10:55:37 dgp85 Exp $
+ * $Id: xine.c,v 1.335 2006/10/16 22:18:24 valtri Exp $
  */
 
 /*
@@ -500,6 +500,7 @@ xine_stream_t *xine_stream_new (xine_t *this,
   stream->spu_channel_user       = -1;
   stream->spu_channel            = -1;
   stream->early_finish_event     = 0;
+  stream->delay_finish_event     = 0;
   stream->gapless_switch         = 0;
 
   stream->video_out              = vo;
@@ -533,6 +534,7 @@ xine_stream_t *xine_stream_new (xine_t *this,
   pthread_mutex_init (&stream->info_mutex, NULL);
   pthread_mutex_init (&stream->meta_mutex, NULL);
   pthread_mutex_init (&stream->demux_lock, NULL);
+  pthread_mutex_init (&stream->demux_mutex, NULL);
   pthread_mutex_init (&stream->event_queues_lock, NULL);
   pthread_mutex_init (&stream->counter_lock, NULL);
   pthread_cond_init  (&stream->counter_changed, NULL);
@@ -574,9 +576,24 @@ xine_stream_t *xine_stream_new (xine_t *this,
    * alloc fifos, init and start decoder threads
    */
 
-  _x_video_decoder_init (stream);
+  if (!_x_video_decoder_init (stream))
+  {
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
 
-  _x_audio_decoder_init (stream);
+  if (!_x_audio_decoder_init (stream))
+  {
+    _x_video_decoder_shutdown(stream);
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
 
   /*
    * osd
@@ -590,7 +607,17 @@ xine_stream_t *xine_stream_new (xine_t *this,
    * create a reference counter
    */
   stream->refcounter = _x_new_refcounter(stream, (refcounter_destructor)xine_dispose_internal);
-  
+  if (!stream->refcounter)
+  {
+    _x_video_decoder_shutdown(stream);
+    _x_audio_decoder_shutdown(stream);
+    free(stream->audio_decoder_extra_info);
+    free(stream->current_extra_info);
+    free(stream);
+    pthread_mutex_unlock(&this->streams_lock);
+    return NULL;
+  }
+
   /*
    * register stream
    */
@@ -617,7 +644,7 @@ static void mrl_unescape(char *mrl) {
 
   for (i = 0; i < len; i++) {
     if ((mrl[i]=='%') && (i<(len-2))) {
-      int c;
+      unsigned int c;
       
       if (sscanf(&mrl[i + 1], "%02x", &c) == 1) {
 	mrl[i]= (char)c;
@@ -700,6 +727,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
      */
 
     if ((stream->input_plugin = _x_find_input_plugin (stream, input_source))) {
+      int res;
+
       xine_log (stream->xine, XINE_LOG_MSG, _("xine: found input plugin  : %s\n"),
 		stream->input_plugin->input_class->get_description(stream->input_plugin->input_class));
       if (stream->input_plugin->input_class->eject_media)
@@ -707,15 +736,23 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
       _x_meta_info_set_utf8(stream, XINE_META_INFO_INPUT_PLUGIN, 
 			    (stream->input_plugin->input_class->get_identifier (stream->input_plugin->input_class)));
 
-      if (!stream->input_plugin->open(stream->input_plugin)) {
+      res = stream->input_plugin->open(stream->input_plugin);
+      switch(res) {
+      case 1: /* Open successfull */
+	free(input_source);
+	break;
+      case -1: /* Open unsuccessfull, but correct plugin */
+	free(input_source);
+	stream->err = XINE_ERROR_INPUT_FAILED;
+	_x_flush_events_queues (stream);
+	return 0;
+      default:
 	xine_log (stream->xine, XINE_LOG_MSG, _("xine: input plugin cannot open MRL [%s]\n"),mrl);
 	stream->input_plugin->dispose(stream->input_plugin);
 	stream->input_plugin = NULL;
 	stream->err = XINE_ERROR_INPUT_FAILED;
-      } else {
-        free(input_source);
-        break;
       }
+      if ( res ) break;
     }
 
     free(input_source);
@@ -790,6 +827,8 @@ static int open_internal (xine_stream_t *stream, const char *mrl) {
 
 	  xine_log(stream->xine, XINE_LOG_MSG, _("xine: join rip input plugin\n"));
 	  input_saver = _x_rip_plugin_get_instance (stream, filename);
+	  free(filename);
+	  
 	  if( input_saver ) {
 	    stream->input_plugin = input_saver;
 	  } else {
@@ -1225,6 +1264,8 @@ int xine_play (xine_stream_t *stream, int start_pos, int start_time) {
 
   pthread_mutex_lock (&stream->frontend_lock);
 
+  stream->delay_finish_event = 0;
+  
   ret = play_internal (stream, start_pos, start_time);
   if( stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_PLAY) )
     xine_play (stream->slave, start_pos, start_time);
@@ -1270,6 +1311,7 @@ void xine_dispose_internal (xine_stream_t *stream) {
   pthread_mutex_destroy (&stream->event_queues_lock);
   pthread_mutex_destroy (&stream->current_extra_info_lock);
   pthread_cond_destroy  (&stream->counter_changed);
+  pthread_mutex_destroy (&stream->demux_mutex);
   pthread_mutex_destroy (&stream->demux_lock);
   pthread_mutex_destroy (&stream->first_frame_lock);
   pthread_cond_destroy  (&stream->first_frame_reached);
@@ -1330,7 +1372,8 @@ void xine_exit (xine_t *this) {
   xprintf (this, XINE_VERBOSITY_DEBUG, "xine_exit: bye!\n");
 
   for (i = 0; i < XINE_LOG_NUM; i++)
-    this->log_buffers[i]->dispose (this->log_buffers[i]);
+    if ( this->log_buffers[i] )
+      this->log_buffers[i]->dispose (this->log_buffers[i]);
 
   _x_dispose_plugins (this);
 
@@ -1357,7 +1400,6 @@ void xine_exit (xine_t *this) {
 
 xine_t *xine_new (void) {
   xine_t      *this;
-  int          i;
 
 #ifdef WIN32
     WSADATA Data;
@@ -1391,9 +1433,7 @@ xine_t *xine_new (void) {
   /*
    * log buffers
    */
-
-  for (i = 0; i < XINE_LOG_NUM; i++)
-    this->log_buffers[i] = _x_new_scratch_buffer (150);
+  memset(this->log_buffers, 0, sizeof(this->log_buffers));
 
 
 #ifdef WIN32
@@ -1552,9 +1592,9 @@ void xine_init (xine_t *this) {
   this->config->register_num(this->config,
       "media.network.timeout", 30,
       _("Timeout for network stream reading (in seconds)"),
-      _("Specifies the timeout when reading from network streams, in seconds."
-	"Too low values might stop streaming when the source is slow or the"
-	"bandwidth is occupied, too high values will freeze the player if the"
+      _("Specifies the timeout when reading from network streams, in seconds. "
+	"Too low values might stop streaming when the source is slow or the "
+	"bandwidth is occupied, too high values will freeze the player if the "
 	"connection is lost."),
       0, NULL, this);
 
@@ -1915,6 +1955,9 @@ void xine_log (xine_t *this, int buf, const char *format, ...) {
   va_list argp;
   char    buffer[SCRATCH_LINE_LEN_MAX];
   
+  if ( ! this->log_buffers[buf] )
+    this->log_buffers[buf] = _x_new_scratch_buffer(150);
+
   va_start (argp, format);
   this->log_buffers[buf]->scratch_printf (this->log_buffers[buf], format, argp);
   va_end(argp);
@@ -1927,12 +1970,24 @@ void xine_log (xine_t *this, int buf, const char *format, ...) {
   }  
 }
 
+void xine_vlog(xine_t *this, int buf, const char *format, 
+                va_list args)
+{
+  if ( ! this->log_buffers[buf] )
+    this->log_buffers[buf] = _x_new_scratch_buffer(150);
+
+  this->log_buffers[buf]->scratch_printf(this->log_buffers[buf], format, args);
+}
+
 const char *const *xine_get_log (xine_t *this, int buf) {
 
   if(buf >= XINE_LOG_NUM)
     return NULL;
 
-  return this->log_buffers[buf]->get_content (this->log_buffers[buf]);
+  if ( this->log_buffers[buf] )
+    return this->log_buffers[buf]->get_content (this->log_buffers[buf]);
+  else
+    return NULL;
 }
 
 void xine_register_log_cb (xine_t *this, xine_log_cb_t cb, void *user_data) {
